package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IProductionRecordReportExtDao;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesProductionRecordReportExtModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 加工记录合并批量汇报查询报表
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2024/9/29 10:21
 * @Modify:
 **/
@Service
public class ProductionRecordReportExtDao implements IProductionRecordReportExtDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesProductionRecordReportExtModel> queryProductionRecordUnionWorkOrderLogReport(MesProductionRecordReportExtModel model, Pager pager, List<Integer> reportStatusList) {

        List<MesProductionRecordReportExtModel> resultList = new ArrayList<>();

        StringBuilder queryHql = new StringBuilder();
        StringBuilder countHql = new StringBuilder();

        StringBuilder unionQueryHql = new StringBuilder();
        StringBuilder countQueryHql = new StringBuilder();
        StringBuilder prQueryHql = new StringBuilder();
        StringBuilder prCommonQueryHql = new StringBuilder();
        StringBuilder woQueryHql = new StringBuilder();
        StringBuilder woCommonQueryHql = new StringBuilder();

        if (!CollectionUtils.isEmpty(reportStatusList) && model.getIsQueryWo()) {
            unionQueryHql.append("SELECT\n" +
                    "\to.workCenterCode,\n" +
                    "\to.workCellCode,\n" +
                    "\to.partNo,\n" +
                    "\to.partName,\n" +
                    "\to.shiftCode,\n" +
                    "\to.equipmentCode,\n" +
                    "\to.equipmentName,\n" +
                    "\to.reportStatus,\n" +
                    "\to.dataSource,\n" +
                    "\to.statisticsQty \n" +
                    "FROM\n" +
                    "\t(\n" +
                    "\t\t(\n");

            countQueryHql.append("SELECT count(1)\n" +
                    "FROM\n" +
                    "\t(\n" +
                    "\t\t(\n");
        } else {
            countQueryHql.append("SELECT count(1) FROM\n");
        }

        if (!CollectionUtils.isEmpty(reportStatusList)) {
            prQueryHql.append("\t\tSELECT\n" +
                    "\t\t\ta.work_center_code AS workCenterCode,\n" +
                    "\t\t\ta.work_cell_code AS workCellCode,\n" +
                    "\t\t\ta.part_no AS partNo,\n" +
                    "\t\t\ta.part_name AS partName,\n" +
                    "\t\t\ta.shift_code AS shiftCode,\n" +
                    "\t\t\ta.equipment_code AS equipmentCode,\n" +
                    "\t\t\ta.equipment_name AS equipmentName,\n" +
                    "\t\t\ta.report_status AS reportStatus,\n" +
                    "\t\t\t1 AS dataSource,\n" +
                    "\t\t\tSUM( qty ) AS statisticsQty \n" +
                    "\t\tFROM\n");

            prCommonQueryHql.append("\t\t\tmes_production_record a \n" +
                    "\t\tWHERE\n" +
                    "\t\t\ta.organize_code = :organizeCode \n" +
                    "\t\t\tAND a.is_deleted = :isDeleted \n" +
                    "\t\t\tAND a.is_valid = :isValid \n" +
                    "\t\t\tAND a.complete_date_time >= :completeDateTimeStart \n" +
                    "\t\t\tAND a.complete_date_time <= :completeDateTimeEnd \n");
            if (reportStatusList.size() == 1) {
                prCommonQueryHql.append("\t\t\tAND a.report_status = :reportStatus \n");
            } else {
                prCommonQueryHql.append("\t\t\tAND a.report_status in (:reportStatus) \n");
            }
            if (!StringUtils.isEmpty(model.getPartNo())) {
                prCommonQueryHql.append("\t\t\tAND a.part_no = :partNo \n");
            }
            if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
                prCommonQueryHql.append("\t\t\tAND a.work_center_code = :workCenterCode \n");
            }
            if (!StringUtils.isEmpty(model.getWorkCellCode())) {
                prCommonQueryHql.append("\t\t\tAND a.work_cell_code = :workCellCode \n");
            }
            if (!StringUtils.isEmpty(model.getShiftCode())) {
                prCommonQueryHql.append("\t\t\tAND a.shift_code = :shiftCode \n");
            }
            if (!StringUtils.isEmpty(model.getEquipmentCode())) {
                prCommonQueryHql.append("\t\t\tAND a.equipment_code = :equipmentCode \n");
            }
            prCommonQueryHql.append("\t\tGROUP BY\n" +
                    "\t\t\ta.work_center_code,\n" +
                    "\t\t\ta.work_cell_code,\n" +
                    "\t\t\ta.part_no,\n" +
                    "\t\t\ta.shift_code,\n" +
                    "\t\t\ta.report_status \n" +
                    "\t\tORDER BY\n" +
                    "\t\t\ta.complete_date_time DESC \n");
        }

        if (!CollectionUtils.isEmpty(reportStatusList) && model.getIsQueryWo()) {
            prCommonQueryHql.append("\t\t) UNION ALL\n" +
                    "\t\t(");
        }

        if (model.getIsQueryWo()) {
            woQueryHql.append("\t\tSELECT\n" +
                    "\t\t\tb.work_center_code AS workCenterCode,\n" +
                    "\t\t\tNULL AS workCellCode,\n" +
                    "\t\t\tb.part_no AS partNo,\n" +
                    "\t\t\tb.part_name AS partName,\n" +
                    "\t\t\tb.shift_code AS shiftCode,\n" +
                    "\t\t\tNULL AS equipmentCode,\n" +
                    "\t\t\tNULL AS equipmentName,\n" +
                    "\t\t\t30 AS reportStatus,\n" +
                    "\t\t\t2 AS dataSource,\n" +
                    "\t\t\tSUM( modify_qty ) AS statisticsQty \n" +
                    "\t\tFROM\n");

            woCommonQueryHql.append("\t\t\tmes_work_order_log b \n" +
                    "\t\tWHERE\n" +
                    "\t\t\tb.organize_code = :organizeCode \n" +
                    "\t\t\tAND b.is_deleted = :isDeleted \n" +
                    "\t\t\tAND b.is_valid = :isValid \n" +
                    "\t\t\tAND b.modify_date_time >= :completeDateTimeStart \n" +
                    "\t\t\tAND b.modify_date_time <= :completeDateTimeEnd \n" +
                    "\t\t\tAND b.type = :messageType \n");
            if (!StringUtils.isEmpty(model.getPartNo())) {
                woCommonQueryHql.append("\t\t\tAND b.part_no = :partNo \n");
            }
            if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
                woCommonQueryHql.append("\t\t\tAND b.work_center_code = :workCenterCode \n");
            }
            if (!StringUtils.isEmpty(model.getShiftCode())) {
                woCommonQueryHql.append("\t\t\tAND b.shift_code = :shiftCode \n");
            }
            woCommonQueryHql.append("\t\tGROUP BY\n" +
                    "\t\t\tb.work_center_code,\n" +
                    "\t\t\tb.part_no,\n" +
                    "\t\t\tb.shift_code \n" +
                    "\t\tORDER BY\n" +
                    "\t\t\tb.modify_date_time DESC \n");
        }

        if (!CollectionUtils.isEmpty(reportStatusList) && model.getIsQueryWo()) {
            woCommonQueryHql.append("\t) \n" +
                    "\t) AS o \n");
        }

        if (!CollectionUtils.isEmpty(reportStatusList) && model.getIsQueryWo()) {
            queryHql.append(unionQueryHql).append(prQueryHql).append(prCommonQueryHql).append(woQueryHql).append(woCommonQueryHql);
            countHql.append(countQueryHql).append(prQueryHql).append(prCommonQueryHql).append(woQueryHql).append(woCommonQueryHql);
        } else {
            if (!CollectionUtils.isEmpty(reportStatusList)) {
                queryHql.append(prQueryHql).append(prCommonQueryHql);
                countHql.append(countQueryHql).append(prCommonQueryHql);
            } else {
                queryHql.append(woQueryHql).append(woCommonQueryHql);
                countHql.append(countQueryHql).append(woCommonQueryHql);
            }
        }

        Query dataQuery = entityManager.createNativeQuery(queryHql.toString());
        setQueryParams(dataQuery, model, reportStatusList);
        Query countQuery = entityManager.createNativeQuery(countHql.toString());
        setQueryParams(countQuery, model, reportStatusList);

        pager = PagerHelper.getPager(pager, Integer.parseInt(countQuery.getSingleResult().toString()));

        dataQuery.setMaxResults(pager.getPageSize());
        if (pager.getCurrentPage() == 0) {
            pager.setCurrentPage(1);
        }
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());

        List list = dataQuery.getResultList();

        for (Object object : list) {

            if (null == object) continue;

            Object[] objectArr = (Object[]) object;

            if (null == objectArr || objectArr.length != 10) continue;

            MesProductionRecordReportExtModel result = new MesProductionRecordReportExtModel(
                    objectArr[0], objectArr[1], objectArr[2], objectArr[3], objectArr[4], objectArr[5], objectArr[6], objectArr[7], objectArr[8], objectArr[9]);
            if (!StringUtils.isEmpty(result.getDataSource()) && !StringUtils.isEmpty(result.getReportStatus())) {
                if (result.getDataSource().compareTo(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) == 0) {
                    if (result.getReportStatus().compareTo(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getValue()) == 0) result.setReportStatusName(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getDescription());
                    else result.setReportStatusName(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getDescription());
                } else result.setReportStatusName(MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT.getDescription());
            }

            resultList.add(result);
        }

        return new ListPager(resultList, pager);

    }

    private void setQueryParams(Query query, MesProductionRecordReportExtModel model, List<Integer> reportStatusList) {
        query.setParameter(MesExtConstWords.ORGANIZE_CODE, model.getOrganizeCode());
        query.setParameter(MesExtConstWords.IS_DELETED, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter(MesExtConstWords.IS_VALID, CommonEnumUtil.IS_VAILD.VAILD.getValue());
        query.setParameter(MesExtConstWords.COMPLETE_DATE_TIME_START, model.getCompleteDateTimeStart());
        query.setParameter(MesExtConstWords.COMPLETE_DATE_TIME_END, model.getCompleteDateTimeEnd());
        if (!CollectionUtils.isEmpty(reportStatusList)) {
            if (reportStatusList.size() == 1) {
                query.setParameter(MesExtConstWords.REPORT_STATUS, reportStatusList.get(0));
            } else {
                query.setParameter(MesExtConstWords.REPORT_STATUS, reportStatusList);
            }
        }
        if (model.getIsQueryWo()) query.setParameter(MesExtConstWords.MESSAGE_TYPE, MesExtEnumUtil.WORK_ORDER_LOG_TYPE.REPORT.getValue());
        if (!StringUtils.isEmpty(model.getPartNo())) {
            query.setParameter(MesExtConstWords.PART_NO, model.getPartNo());
        }
        if (!StringUtils.isEmpty(model.getWorkCenterCode())) {
            query.setParameter(MesExtConstWords.WORK_CENTER_CODE, model.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(model.getWorkCellCode())) {
            query.setParameter(MesExtConstWords.WORK_CELL_CODE, model.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(model.getShiftCode())) {
            query.setParameter(MesExtConstWords.SHIFT_CODE, model.getShiftCode());
        }
        if (!StringUtils.isEmpty(model.getEquipmentCode())) {
            query.setParameter(MesExtConstWords.EQUIPMENT_CODE, model.getEquipmentCode());
        }
    }

}
